from hatchet_sdk.runnables.workflow import Workflow
from pydantic import BaseModel, HttpUrl, ConfigDict, field_serializer
from fastapi import FastAPI
from typer import Typer


class Plugin(BaseModel):
    id: str
    module: str
    description: str


class APIPlugin(Plugin):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    api: FastAPI

    @field_serializer("api")
    def serialize_workflow(self, api: FastAPI, _info):
        pass


class CLIPlugin(Plugin):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    cli: Typer

    @field_serializer("cli")
    def serialize_workflow(self, cli: Typer, _info):
        pass


def serialize_workflow_config(config):
    return config.model_json_schema()


class WorkflowPlugin(Plugin):
    model_config = ConfigDict(arbitrary_types_allowed=True)

    wf: Workflow

    @field_serializer("wf")
    def serialize_workflow(self, wf: Workflow, _info):
        return {
            "config": wf.config.model_dump(
                exclude_defaults=True, fallback=serialize_workflow_config
            ),
            "id": wf.id,
            "is_durable": wf.is_durable,
            "name": wf.name,
            "service_name": wf.service_name,
            "tasks": [task.name for task in wf.tasks],
        }


class SettingsPlugin(Plugin): ...


class Plugins(BaseModel):
    api: dict[str, APIPlugin] = {}
    cli: dict[str, CLIPlugin] = {}
    workflow: dict[str, WorkflowPlugin] = {}
    settings: dict[str, SettingsPlugin] = {}


class ServerInfo(BaseModel):
    version: str
    hatchet_url: HttpUrl
    api_doc: str
    plugins: Plugins = Plugins()
